#!/usr/bin/python
import sys
import os
import shutil
import os.path
import gtk
import gtk.gdk
import glob
import gobject
from pxeos import *
from netboot_util import *
from netboot_gtk import *

class disklessDialog:
    def __init__(self,xml):
        self.xml=xml
        self.dlg = self.xml.get_widget("disklessDruidDialog")
        self.druid = self.xml.get_widget("disklessDruid")
        self.page1 = self.xml.get_widget("druidpagestart1")
        self.druid.connect("cancel",self.cancel)
        self.dirEntry=self.xml.get_widget("disklessDirEntry")
        self.serverEntry=self.xml.get_widget("disklessServerEntry")
        self.nameEntry=self.xml.get_widget("disklessNameEntry")
        self.descriptionEntry=self.xml.get_widget("disklessDescriptionEntry")
        self.xml.get_widget("disklessNamePage").connect("next",self.on_namePage_next)
        self.xml.get_widget("disklessDirPage").connect("next",self.on_dirPage_next)
        self.xml.get_widget("disklessDruidFinish").connect("finish",self.on_finish)
        self.kernelTreeview=self.xml.get_widget("kernelTreeview")
        model = gtk.ListStore (gobject.TYPE_STRING)
        self.kernelTreeview.set_model (model)
        column = gtk.TreeViewColumn ("", gtk.CellRendererText(), text=0)
        self.kernelTreeview.append_column (column)
        self.OS=None
        self.tmpdir=None

    def cancel(self,button):
        self.OS.unmount()
        self.dlg.response(gtk.RESPONSE_CANCEL)

    def run(self,OS):
        self.OS=OS
        if self.OS.getName()=="":
            self.OS.setProtocol("NFS")
            self.OS.setDiskless(TRUE)
            self.nameEntry.set_sensitive(TRUE)
        else:
            self.nameEntry.set_sensitive(FALSE)
        self.descriptionEntry.set_text(self.OS.getDescription())
        self.nameEntry.set_text(self.OS.getName())
        self.nameEntry.grab_focus()
        self.serverEntry.set_text(self.OS.getServer())
        self.dirEntry.set_text(self.OS.getLocation())
        self.druid.set_page(self.page1)
        self.druid.show_all()
        rc=self.dlg.run()
        self.dlg.hide()
        return rc,self.OS

    def on_finish(self,*args):
        self.dir=self.dirEntry.get_text()
        model=self.kernelTreeview.get_model()
        result = self.kernelTreeview.get_selection().get_selected ()
        if result == None:
            return TRUE
        else:
            (model, iter) = result
            try:        
                self.kernel = model.get_value (iter, 0)
            except:
                return TRUE
        cursor=Cursor(self.dlg.window)
        cursor.wait()
        self.pxedir=self.OS.getPXEDir()
        self.OS.setKernel(self.kernel)
        self.OS.makeDiskless()
        rc=self.OS.updatePXE()
        cursor.ready()
        if rc[0] != 0:
            errorMessage(rc[1])
            return TRUE
        self.dlg.response(gtk.RESPONSE_OK)

        
    def on_namePage_next(self,*args):
        name=self.nameEntry.get_text()
        if name == "":
            errorMessage(_("You must enter a name"))
            return TRUE
        self.OS.setName(name)
        self.OS.setDescription(self.descriptionEntry.get_text())
        self.dirEntry.grab_focus()

    def on_dirPage_next(self,*args):
        selIter=None
        server=self.serverEntry.get_text()
        try:
            socket.gethostbyname(server)
        except socket.gaierror,e:
            errorMessage(_("Must be a valid hostname or ipaddress"))
            return TRUE
            
        self.OS.unmount()
        location=self.dirEntry.get_text()
        self.OS.setLocation(location)
        self.OS.setServer(server)
        self.OS.mount()
        l=self.OS.getKernels()
        if len(l) == 0:
            errorMessage(_("The diskless directory must be NFS exported and contain a boot sub-directory"))
            self.OS.unmount()
            return TRUE
        l.sort()
        model=self.kernelTreeview.get_model()
        model.clear()
        for i in l:
            x=i.split("vmlinuz-")
            iter=model.append()
            model.set_value(iter,0,x[1])
            if x[1]==self.OS.getKernel():
                selIter=iter
        selection=self.kernelTreeview.get_selection()
        if selIter==None:
            selection.select_path ((0,))
        else:
            selection.select_iter (selIter)

def copyOS(host,destdir):
    TOPROOT="%s/root" % destdir
    mkdir (TOPROOT)
    os.system ("cd %s; rsync -a -e ssh --exclude /proc --exclude /diskless root@%s:/ ." % (TOPROOT,host))

def linkdir(SRCDIR,DESTDIR):
    mkdir(DESTDIR)
    for i in os.listdir(SRCDIR):
        src="%s/%s" % (SRCDIR ,i)
        dest="%s/%s" % (DESTDIR,i)
        if os.path.islink(src):
            os.system("ln -f %s %s 2> /dev/null" % (src,dest))
        else:
            if os.path.isdir(src):
                linkdir(src,dest)
            else:
                if os.path.exists(dest):
                    os.unlink(dest)
                os.link(src,dest)

class Diskless:
    def __init__(self,ip,toproot,swapdisk=""):
        self.ip=ip
        self.swapdisk=swapdisk
        self.toprootdir=toproot
        
    def delete(self):
        os.system("rm -rf %s/%s" % (self.toproot,self.ip))
        
